---

# If you need to change the project name
# project_name: my_project

# Used in some images, dev by default
# environment: dev


# Proxy enabled by default (with traefik)
# proxy:
#   enabled: true
#   domain: localhost
#   port: 80

# Block certain ports for certain images
# For now, only PHP is supported
# network-block:
#   php: [25, 465, 587]

# List of services with version and other options
# services:
#   adminer:
#     enabled: false
#     version: latest
#     ram: 256M
#
#   apache:
#     enabled: false
#     version: latest
#     ram: 512M
#
#   elasticsearch:
#     enabled: false
#     version: latest
#     ram: 2048M
#
#   kibana:
#     enabled: false
#     version: latest
#     ram: 2048M
#
#   logstash:
#     enabled: false
#     version: latest
#     ram: 2048M
#
#   mailcatcher:
#     enabled: false
#     version: latest
#     ram: 256M
#
#   maildev:
#     enabled: false
#     version: latest
#     ram: 256M
#
#   mongo:
#     enabled: false
#     version: latest
#     ram: 1024M
#
#   mysql:
#     enabled: false
#     version: latest
#     ram: 1024M
#     root_password: root
#
#   nginx:
#     enabled: false
#     version: latest
#     ram: 1024M
#     document_root: /var/www
#
#   php:
#     enabled: false
#     version: latest
#     ram: 1024M
#
#   phpmyadmin:
#     enabled: false
#     version: latest
#     ram: 256M
#     upload_max_filesize: 256M
#
#   portainer:
#     enabled: false
#     version: latest
#     ram: 512M
#
#   postgres:
#     enabled: false
#     version: latest
#     ram: 1024M
#     password: postgres
#
#   sqlserver:
#     enabled: false
#     version: latest
#     ram: 1024M
#     password: postgres
#
#   redis:
#     enabled: false
#     version: latest
#     ram: 1024M
#
#   xhgui:
#     enabled: false
#     version: latest
#     ram: 1024M
#
